------------------------------------------------------------------
-- Update records in a table in bulk using Forall
------------------------------------------------------------------
declare
  cn_bulk_limit  constant  integer  := 1000;

  cursor c_err
  is
  select rec_id
  from   bulk_errors_perf
  ;

  type ta_err is table of c_err%rowtype  index by pls_integer;
  a_err     ta_err;
  
  l_count   integer  := 0;
  l_time    number   := 0;
  l_start   number;
begin
  open  c_err;
  l_start := dbms_utility.get_time;
  loop
    fetch c_err
    bulk collect
    into  a_err
    limit cn_bulk_limit;

    exit when a_err.count = 0;

    l_start := dbms_utility.get_time;
    forall i_err in indices of a_err
      update bulk_errors_perf
      set    cust_last_name = trim(cust_last_name)
      where  rec_id         = a_err(i_err).rec_id
    ;
      
    commit;
    l_time := l_time + (dbms_utility.get_time - l_start);
    l_count := l_count + a_err.count;

  end loop;

  dbms_output.put_line ('Runtime (sec)  : '||to_char(l_time/100,'fm99999990d00','NLS_NUMERIC_CHARACTERS = '',.'''));
    
  close c_err;

  dbms_output.put_line ('Records updated: '||to_char(l_count,'fm999g999g999','NLS_NUMERIC_CHARACTERS = '',.'''));
end;
/
