declare
  subtype st_maxchar         is varchar2(32767);
  subtype st_identifier      is user_objects.object_name%type;
  subtype st_object_type     is user_objects.object_type%type;
  
  cn_type_table     constant  st_object_type  := 'table'   ;
  cn_type_sequence  constant  st_object_type  := 'sequence';
  cn_type_type      constant  st_object_type  := 'type'    ;
  
  procedure silent_drop
  (p_object_type  in  st_identifier
  ,p_object_name  in  st_identifier
  )
  is
    l_stmnt   st_maxchar;
    e_missing_table      exception;
    e_missing_sequence   exception;
    e_missing_type       exception;
    pragma exception_init (e_missing_table   ,-942 );
    pragma exception_init (e_missing_sequence,-2289);
    pragma exception_init (e_missing_type    ,-4043);
  begin
    l_stmnt := 'drop '||p_object_type||' '||p_object_name;
    execute immediate l_stmnt;
  exception
    when e_missing_table   
      or e_missing_sequence
      or e_missing_type  
    then
      -- I want to get rid of it, so I don't care if it's already gone
      null;   
  end silent_drop;
  
begin
  silent_drop (cn_type_table   ,'bulk_errors_perf');
  silent_drop (cn_type_sequence,'bulk_errors_perf_seq1');
  silent_drop (cn_type_type    ,'err_perf_ntt');
  silent_drop (cn_type_type    ,'err_perf_ot');
  silent_drop (cn_type_type    ,'err_perf_ot');
end;
/


set timing on

------------------------------------------------------------------------ Sequence bulk_errors_perf_seq1
create sequence bulk_errors_perf_seq1
minvalue 1000000
start with 1000000
increment by 1
order;


------------------------------------------------------------------------ Table bulk_errors_perf
create table bulk_errors_perf
as 
with multiplier
  as (select level   iteration
      from   dual
      connect by level <= 20   --200
     )
select bulk_errors_perf_seq1.nextval        rec_id
,      t.*
from   sh.customers    t
cross join multiplier
;


create index bulk_errors_perf_ind1
on  bulk_errors_perf (rec_id);


select count(*)  rows_in_perf_table
from   bulk_errors_perf
;


------------------------------------------------------------------------ Object Type err_perf_ot
create or replace type err_perf_ot as object
 (rec_id      number
 
 ,constructor function err_perf_ot return self as result
 )
/

create or replace type body err_perf_ot is
  constructor function err_perf_ot
  return self as result
  is   
  begin
    self.rec_id     := null;
    
    return;
  end;
end;
/

------------------------------------------------------------------------ Object Type t_err_perf
create or replace type err_perf_ntt as table of err_perf_ot
/


set timing off
