------------------------------------------------------------------
-- drop objects if they exist
------------------------------------------------------------------
drop table ero_demo_sparse_status_coll;

------------------------------------------------------------------
-- Create table to perform DML on
------------------------------------------------------------------
create table ero_demo_sparse_status_coll
(id       number(15,0)   not null
,value    number
);

------------------------------------------------------------------
-- Create a constraint that will cause errors
------------------------------------------------------------------
alter table ero_demo_sparse_status_coll
add (constraint ero_demo_sparse_status_c_ck1
     check (-- Value must be more than 0
            value > 0
           )
    );


------------------------------------------------------------------
-- Mark rejected records as such using an extra collection attribute
------------------------------------------------------------------
declare
  c_process  constant  integer  := 1;
  c_reject   constant  integer  := 0;
  
  
  cursor l_val_cur
  is
  select level * 111            id
  ,      100 * mod(level , 3)   value
  ,      c_process              action
  from   dual
  connect by level <= 10
  ;
  
  type l_val_aat is table of l_val_cur%rowtype  index by pls_integer;
  l_val_aa     l_val_aat;
  
  l_error_index    integer;
  l_error_code     integer;
  l_index          pls_integer;
  
  e_bulk_errors    exception;
  pragma exception_init(e_bulk_errors, -24381);
begin
  -- populate collection
  open  l_val_cur;
  fetch l_val_cur
  bulk collect
  into  l_val_aa;
  close l_val_cur;
  
  -- for some reason we want to disregard records 4 and 8
  l_val_aa(4).action := c_reject;
  l_val_aa(8).action := c_reject;
  
  -- List contents of collection
  dbms_output.put_line ('contents of collection');
  dbms_output.put_line ('======================');
  dbms_output.put_line ('Index ID    Value Action');
  dbms_output.put_line ('===== ===== ===== =======');
  l_index := l_val_aa.first;
  while l_index is not null
  loop
    dbms_output.put_line (lpad(to_char(l_index                ,'fm99999'),5)||' '||
                          lpad(to_char(l_val_aa(l_index).id   ,'fm99999'),5)||' '||
                          lpad(to_char(l_val_aa(l_index).value,'fm99999'),5)||' '||
                          case l_val_aa(l_index).action
                            when c_process then 'Process'
                            when c_reject  then 'Reject'
                            else                to_char(l_val_aa(l_index).action)||' ??'
                          end 
                         );
    l_index := l_val_aa.next (l_index);
  end loop;
  dbms_output.put_line ('');
  
  
  -- Do bulk data manipulation
  begin
    forall i_val in indices of l_val_aa  save exceptions
      insert 
      into   ero_demo_sparse_status_coll
             (id
             ,value
             )
      select l_val_aa(i_val).id   
      ,      l_val_aa(i_val).value
      from   dual
      where  l_val_aa(i_val).action = c_process
    ;
  exception
    when e_bulk_errors
    then
      dbms_output.put_line ('Exceptions');
      dbms_output.put_line ('==========');
      
      for i_err in 1 .. sql%bulk_exceptions.count
      loop
        l_error_index := sql%bulk_exceptions(i_err).error_index;
        l_error_code  := sql%bulk_exceptions(i_err).error_code ;
        
        dbms_output.put_line ('Exception sequence: '||to_char(i_err)                        );
        dbms_output.put_line ('Error_index       : '||to_char(l_error_index)                );
        dbms_output.put_line ('Error_code        : '||to_char(l_error_code)                 );
        dbms_output.put_line ('Error Message     : '||sqlerrm(-1 * l_error_code)            );
        dbms_output.put_line ('ID                : '||to_char(l_val_aa(l_error_index).id)   );
        dbms_output.put_line ('Value             : '||to_char(l_val_aa(l_error_index).value));
        
        dbms_output.put_line ('');
      end loop;
  end;
    
  commit;

end;
/

------------------------------------------------------------------
-- Query the table
------------------------------------------------------------------
select * 
from   ero_demo_sparse_status_coll;

