------------------------------------------------------------------
-- drop objects if they exist
------------------------------------------------------------------
drop  table tst_bulk_triggers;


------------------------------------------------------------------
-- Create table to perform DML on
------------------------------------------------------------------
create table tst_bulk_triggers
(id     number(15,0)    not null
,num    number(15,0)    not null
);


------------------------------------------------------------------
-- Create a compound trigger on table to monitor trigger timings
------------------------------------------------------------------
create or replace trigger tst_cmp
for insert or update or delete
on tst_bulk_triggers
compound trigger

  gc_action  constant   varchar2(30)  := case
                                           when inserting then 'INSERT'
                                           when updating  then 'UPDATE'
                                           when deleting  then 'DELETE'
                                           else                'UNKNOWN'
                                         end;

  before statement
    is
    begin
      dbms_output.put_line ('BEFORE STATEMENT '||gc_action||' TRIGGER');
    end before statement;

  before each row
    is
    begin
      dbms_output.put_line ('BEFORE ROW '||gc_action||' TRIGGER');
    end before each row;

  after each row
    is
    begin
      dbms_output.put_line ('AFTER ROW '||gc_action||' TRIGGER');
    end after each row;

  after statement
    is
    begin
      dbms_output.put_line ('AFTER STATEMENT '||gc_action||' TRIGGER');
    end after statement;
end tst_cmp;
/


------------------------------------------------------------------
-- Perform Insert, Update, Merge and Delete with Forall
------------------------------------------------------------------
declare
  TYPE tr_tst IS RECORD (id  number(15));
  r_tst     tr_tst;
  TYPE ta_tst IS TABLE OF tr_tst INDEX BY PLS_INTEGER;
  a_tst    ta_tst;
  

  PROCEDURE pl
  (p_line  IN  VARCHAR2  DEFAULT ''
  )
  IS
  BEGIN
    dbms_output.put_line (p_line);
  END pl;

begin
  for i in 1 .. 4
  loop
    a_tst(i).id := i;
  end loop;
  
  pl ('INSERT:');
  pl ('######');
  forall i in indices of a_tst
    insert
    into   tst_bulk_triggers
           (id
           ,num
           )
    values (a_tst(i).id
           ,a_tst(i).id
           )
  ;
  commit;
  pl;
  
  pl ('UPDATE:');
  pl ('######');
  forall i in indices of a_tst
    update tst_bulk_triggers
    set    num = a_tst(i).id + 1
    where  id  = a_tst(i).id
  ;
  commit;
  pl;

  pl ('MERGE:');
  pl ('######');
  forall i in indices of a_tst
    MERGE 
    INTO   tst_bulk_triggers   tgt
    USING (SELECT a_tst(i).id    id
           FROM   dual
          )    src
    ON    (tgt.id = src.id
          )
    WHEN MATCHED 
    THEN
      UPDATE
      SET    tgt.num = src.id
    WHEN NOT MATCHED 
    THEN
      INSERT (tgt.id
             ,num
             )
      VALUES (src.id
             ,src.id
             )
  ;
  commit;
  pl;

  pl ('DELETE:');
  pl ('######');
  forall i in indices of a_tst
    delete
    from   tst_bulk_triggers
    where  id  = a_tst(i).id
  ;
  commit;
  pl;

end;
/
